import socket
import logging
import threading
import time
from .utils import ProxyUtils
from .mqtt_client import MQTTClient
from commands.command_heartbeat import CommandHeartbeat
from commands.command_02 import Command02
from commands.command_03 import Command03
from commands.command_07 import Command07
from commands.command_08 import Command08
from commands.command_09 import Command09
from commands.command_0A import Command0A
from commands.command_25 import Command25
from commands.command_30 import Command30
from commands.command_19_1A import Command191A
from commands.command_1F_20 import Command1F20
from commands.command_21_22 import Command2122
from commands.command_23_24 import Command2324
from commands.command_26_27 import Command2627




class ChargingPileProxyServer:
    def __init__(self, listen_host='0.0.0.0', listen_port=52461,
                 forward_host='139.9.209.227', forward_port=52461):
        self.listen_host = listen_host
        self.listen_port = listen_port
        self.forward_host = forward_host
        self.forward_port = forward_port
        self.server_socket = None
        self.running = False
        self.clients = {}
        self.remote_connections = {}
        self.mqtt_client = MQTTClient()
        self.pile_ids = {}
        self.utils = ProxyUtils()
        self.heartbeat_handler = CommandHeartbeat()
        self.command_02_handler = Command02()
        self.command_03_handler = Command03()
        self.command_07_handler = Command07()
        self.command_08_handler = Command08()
        self.command_09_handler = Command09()
        self.command_0a_handler = Command0A()
        self.charge_info_handler = Command25()
        self.bms_handler = Command30()
        self.card_auth_handler = Command191A()
        self.start_charge_handler = Command1F20()
        self.charge_result_handler = Command2122()
        self.order_handler = Command2324()
        self.stop_charge_handler = Command2627()

        # 存储登录信息的字典，以桩号为键
        self.login_info = {}
        # 存储对时信息的字典
        self.time_sync_info = {}
        # 存储遥信信息的字典
        self.remote_signal_info = {}
        # 存储故障信息的字典
        self.fault_info = {}
        # 存储心跳信息的字典
        self.heartbeat_info = {}

        self.heartbeat_thread = threading.Thread(target=self.check_heartbeat_timeout)
        self.heartbeat_thread.daemon = True
        self.heartbeat_thread.start()

    def check_heartbeat_timeout(self):
        """心跳超时检测"""
        while self.running:
            try:
                current_time = time.time()
                disconnected_piles = []

                for pile_id, last_heartbeat in self.heartbeat_info.items():
                    time_diff = current_time - last_heartbeat

                    # 如果超过60秒没收到心跳
                    if time_diff > 60:
                        msg = f"充电桩 {pile_id.hex().upper()} 心跳超时 {time_diff:.1f}秒"
                        logging.warning(msg)
                        print(msg)
                        self.mqtt_client.publish_message(msg)
                        disconnected_piles.append(pile_id)

                # 从心跳信息中移除断开的充电桩
                for pile_id in disconnected_piles:
                    del self.heartbeat_info[pile_id]
                    self.handle_pile_disconnect(pile_id)

                # 每5秒检查一次
                time.sleep(5)

            except Exception as e:
                logging.error(f"心跳检测错误: {str(e)}")
                time.sleep(5)

    def update_heartbeat(self, pile_id):
        """更新心跳时间戳"""
        try:
            self.heartbeat_info[pile_id] = time.time()
            logging.debug(f"更新充电桩 {pile_id.hex().upper()} 心跳时间")
        except Exception as e:
            logging.error(f"更新心跳时间戳失败: {str(e)}")

    def handle_pile_disconnect(self, pile_id):
        """处理充电桩断开连接"""
        try:
            msg = f"充电桩 {pile_id.hex().upper()} 断开连接"
            logging.info(msg)
            print(msg)
            self.mqtt_client.publish_message(msg)

            # 可以在这里添加充电桩断开后的清理逻辑
            # 例如：中断正在进行的充电等

        except Exception as e:
            logging.error(f"处理充电桩断开连接失败: {str(e)}")

    def start(self):
        """启动代理服务器"""
        try:
            self.mqtt_client.connect()

            self.server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.server_socket.bind((self.listen_host, self.listen_port))
            self.server_socket.listen(5)
            self.running = True

            start_msg = f"代理服务器已启动，监听地址: {self.listen_host}:{self.listen_port}"
            logging.info(start_msg)
            print(start_msg)
            self.mqtt_client.publish_message(start_msg)

            while self.running:
                client_socket, address = self.server_socket.accept()
                client_msg = f"收到新的客户端连接，地址: {address}"
                logging.info(client_msg)
                print(client_msg)
                self.mqtt_client.publish_message(client_msg)

                client_thread = threading.Thread(target=self.handle_client,
                                              args=(client_socket, address))
                client_thread.daemon = True
                client_thread.start()

        except Exception as e:
            error_msg = f"代理服务器错误: {str(e)}"
            logging.error(error_msg)
            print(error_msg)
            self.mqtt_client.publish_message(error_msg)
        finally:
            if self.server_socket:
                self.server_socket.close()

    def create_remote_connection(self, client_address):
        """创建与远程服务器的连接"""
        try:
            remote_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            remote_socket.connect((self.forward_host, self.forward_port))

            connect_msg = f"已连接到远程服务器 {self.forward_host}:{self.forward_port}"
            logging.info(connect_msg)
            print(connect_msg)
            self.mqtt_client.publish_message(connect_msg)

            self.remote_connections[client_address] = remote_socket
            return remote_socket
        except Exception as e:
            error_msg = f"连接远程服务器失败: {str(e)}"
            logging.error(error_msg)
            print(error_msg)
            self.mqtt_client.publish_message(error_msg)
            return None

    def forward_data(self, source_socket, destination_socket, source_address):
        """转发数据"""
        try:
            while self.running:
                data = source_socket.recv(1024)  # 接收数据
                if not data:
                    break  # 如果没有数据，退出循环

                # 如果接收到的数据长度大于等于14字节，且以'JX'开头
                if len(data) >= 14 and data[0:2] == b'JX':
                    command = data[2]  # 提取命令字节

                    # 获取本地和远程端口信息用于日志
                    source_local, source_remote = self.utils.get_socket_info(source_socket)
                    dest_local, dest_remote = self.utils.get_socket_info(destination_socket)

                    # 根据命令字节处理不同命令
                    if command == 0x01:  # 01H命令
                        logging.info(f"收到01H连接请求命令: {data.hex().upper()}")
                        if self.command_02_handler.process_and_respond(data, destination_socket):
                            logging.info("01H命令处理完成")
                            continue  # 跳过后续转发，避免重复转发

                    elif command == 0x03:  # 03H命令
                        logging.info(f"收到03H登录命令: {data.hex().upper()}")
                        if self.command_03_handler.process_03h(data):
                            logging.info("03H命令处理完成")
                            continue

                    elif command == 0x0C:  # 0CH桩心跳命令
                        # 提取桩号并更新心跳
                        pile_id = data[3:11]
                        self.update_heartbeat(pile_id)
                        logging.info(f"收到0CH心跳命令: {data.hex().upper()}")
                        if self.heartbeat_handler.process_and_respond(data, destination_socket):
                            logging.info("0CH心跳命令处理完成")
                            continue  # 跳过后续转发

                    elif command == 0x19:  # 19H卡鉴权命令
                        logging.info(f"收到19H卡鉴权命令: {data.hex().upper()}")
                        if self.card_auth_handler.process_and_respond(data, destination_socket):
                            logging.info("19H卡鉴权命令处理完成")
                            continue  # 跳过后续转发


                    elif command == 0x1F:  # 1FH启动充电命令

                        logging.info(f"收到1FH启动充电命令: {data.hex()}")

                        if self.start_charge_handler.process_and_respond(data, destination_socket):
                            logging.info("1FH命令处理完成")

                            continue


                    elif command == 0x20:  # 20H启动充电回复

                        logging.info(f"收到20H启动充电回复: {data.hex()}")

                        self.start_charge_handler.parse_20h(data)

                    elif command == 0x21:  # 21H启动充电结果命令
                        logging.info(f"收到21H启动充电结果命令: {data.hex()}")
                        if self.charge_result_handler.process_and_respond(data, destination_socket):
                            logging.info("21H命令处理完成")
                            continue  # 跳过后续转发

                    elif command == 0x23:  # 23H充电订单命令
                        logging.info(f"收到23H充电订单命令: {data.hex().upper()}")
                        if self.order_handler.process_and_respond(data, destination_socket):
                            logging.info("23H充电订单命令处理完成")
                            continue



                    elif command == 0x25:  # 25H充电信息命令

                        logging.info(f"收到25H充电信息命令: {data.hex()}")

                        if self.charge_info_handler.process_25h(data):
                            logging.info("25H命令处理完成")

                    elif command == 0x26:  # 26H停止充电命令
                        logging.info(f"收到26H停止充电命令: {data.hex()}")
                        if self.stop_charge_handler.build_26h_command(data, destination_socket):
                            logging.info("26H命令处理完成")
                            continue

                    elif command == 0x27:  # 27H停止充电回复
                        logging.info(f"收到27H停止充电回复: {data.hex()}")
                        self.stop_charge_handler.parse_27h(data)

                    elif command == 0x30:  # 30H BMS信息命令
                        logging.info(f"收到30H BMS信息命令: {data.hex().upper()}")
                        if self.bms_handler.parse_30h_bms_status(data):
                            logging.info("30H BMS信息命令处理完成")

                    else:
                        # 未知命令，记录日志
                        logging.warning(f"未知命令：{command:02X}，数据内容: {data.hex().upper()}")

                    # 如果数据来自客户端连接，提取桩号
                    if source_socket not in self.remote_connections.values():
                        pile_id = self.utils.extract_pile_id(data)
                        if pile_id:
                            self.pile_ids[source_address] = pile_id

                    # 判断数据发送方向：是发送到远程服务器还是充电桩
                    is_to_remote = destination_socket in self.remote_connections.values()
                    direction = "发送到远程服务器" if is_to_remote else "发送到充电桩"
                    mqtt_direction = "u" if is_to_remote else "d"

                    # 发送数据到目的地
                    destination_socket.send(data)

                    # 记录数据转发日志
                    msg = (f"数据转发成功: {direction} | "
                           f"本地地址: {source_local} | "
                           f"远程地址: {dest_remote} | "
                           f"命令: {command:02X}H | "
                           f"数据长度: {len(data)}")
                    logging.info(msg)
                    self.mqtt_client.publish_message(msg)

                    # 记录完整的数据内容到调试日志
                    logging.debug(f"完整数据内容: {data.hex().upper()}")

                    # 每次数据转发完成后，检查连接状态
                    if not self.running:
                        break

        except ConnectionResetError:
            logging.error(f"连接被重置: {source_remote}")
            self.mqtt_client.publish_message(f"连接被重置: {source_remote}")

        except socket.timeout:
            logging.error(f"连接超时: {source_remote}")
            self.mqtt_client.publish_message(f"连接超时: {source_remote}")

        except Exception as e:
            logging.error(f"转发数据出错: {str(e)}")
            self.mqtt_client.publish_message(f"转发数据出错: {str(e)}")

        finally:
            # 关闭连接
            try:
                if source_socket:
                    source_socket.close()
                if destination_socket:
                    destination_socket.close()
            except Exception as e:
                logging.error(f"关闭连接出错: {str(e)}")

    def handle_client(self, client_socket, client_address):
        """处理客户端连接"""
        try:
            remote_socket = self.create_remote_connection(client_address)
            if not remote_socket:
                client_socket.close()
                return

            forward_thread = threading.Thread(
                target=self.forward_data,
                args=(client_socket, remote_socket, client_address)
            )
            backward_thread = threading.Thread(
                target=self.forward_data,
                args=(remote_socket, client_socket, client_address)
            )

            forward_thread.daemon = True
            backward_thread.daemon = True

            forward_thread.start()
            backward_thread.start()

            forward_thread.join()
            backward_thread.join()

        except Exception as e:
            error_msg = f"处理客户端连接错误: {str(e)}"
            logging.error(error_msg)
            print(error_msg)
            self.mqtt_client.publish_message(error_msg)
        finally:
            if client_address in self.remote_connections:
                self.remote_connections[client_address].close()
                del self.remote_connections[client_address]
            if client_address in self.pile_ids:
                del self.pile_ids[client_address]
            client_socket.close()
            close_msg = f"客户端连接已关闭 {client_address}"
            logging.info(close_msg)
            print(close_msg)
            self.mqtt_client.publish_message(close_msg)

    def stop(self):
        """停止代理服务器"""
        self.running = False
        if self.server_socket:
            self.server_socket.close()

        for remote_socket in self.remote_connections.values():
            remote_socket.close()
        self.remote_connections.clear()
        self.pile_ids.clear()

        self.mqtt_client.disconnect()

        stop_msg = "代理服务器已停止"
        logging.info(stop_msg)
        print(stop_msg)
        self.mqtt_client.publish_message(stop_msg)